;============================================================================

; Quickdraw example
; -----------------

; (represent obj) pops up a window with the graphical representation of 'obj'.
; To get control back, press the mouse button.
;
; try: (represent '(define (fact n) (if (< n 2) 1 (* (fact (- n 1)) n))))

(define (represent obj)

  (define v-offs  0) ; vertical offset of grid
  (define h-offs 20) ; horizontal

  (define grid-v 24) ; vertical spacing between grid lines
  (define grid-h 48) ; horizontal

  (define cons-v 12) ; vertical size of cons cell
  (define cons-h 24) ; horizontal

  (define arrow-head-length 6) ; size of arrows
  (define arrow-head-width  6)
  (define arrow-space       2)

  (define fontsize 12)

  (define (draw-cons-cell x y)
    (let ((v (+ (* y grid-v) v-offs))
          (h (+ (* x grid-h) h-offs)))
      (mac#framerect (mac#rect v h (+ v cons-v) (+ h cons-h)))
      (mac#moveto (+ h (quotient cons-h 2)) v)
      (mac#line 0 (- cons-v 1))))

  (define (draw-car-arrow x y d) ; draw arrow downwards 'd' grid squares
    (let ((v (+ (* y grid-v) v-offs))
          (h (+ (* x grid-h) h-offs)))
      (mac#moveto (+ h (quotient cons-h 4)) (+ v (quotient cons-v 2)))
      (mac#line 0 (- (* d grid-v) (+ (quotient cons-v 2) arrow-space)))
      (mac#line (quotient arrow-head-width 2) (- arrow-head-length))
      (mac#move (- arrow-head-width) 0)
      (mac#line (quotient arrow-head-width 2) arrow-head-length)))

  (define (draw-cdr-arrow x y d) ; draw arrow to the right 'd' grid squares
    (let ((v (+ (* y grid-v) v-offs))
          (h (+ (* x grid-h) h-offs)))
      (mac#moveto (+ h (quotient (* cons-h 3) 4)) (+ v (quotient cons-v 2)))
      (mac#line (- (* d grid-h) (+ (quotient (* cons-h 3) 4) arrow-space)) 0)
      (mac#line (- arrow-head-length) (quotient arrow-head-width 2))
      (mac#move 0 (- arrow-head-width))
      (mac#line arrow-head-length (quotient arrow-head-width 2))))

  (define (draw-nil x y) ; draw nil in cdr of cons cell
    (let ((v (+ (* y grid-v) v-offs))
          (h (+ (* x grid-h) h-offs)))
      (mac#moveto (+ h (quotient cons-h 2)) (+ v (- cons-v 1)))
      (mac#line (- (quotient cons-h 2) 1) (- (- cons-v 1)))))

  (define (object->string obj)
    (let ((port (open-output-string)))
      (write obj port)
      (let ((str (get-output-string port)))
        (close-output-port port)
        str)))

  (define (object-length obj) ; length of object in grid squares
    (cond ((null? obj)
           0)
          ((pair? obj)
           (+ 1 (object-length (cdr obj))))
          (else
           (+ 1 (quotient (+ (mac#stringwidth (object->string obj)) fontsize)
                          grid-h)))))

  (define (initial-profile) 0)
  (define (car-profile p) (if (pair? p) (car p) p))
  (define (cdr-profile p) (if (pair? p) (cdr p) p))

  (define (make-profile len p)
    (define (fit1 len p)
      (if (> len 1)
        (let ((p* (fit1 (- len 1) (cdr-profile p))))
          (cons (car-profile p*) p*))
        (fit2 (+ (car-profile p) 1) p)))
    (define (fit2 y p)
      (if (pair? p)
        (cons (max y (car-profile p)) (fit2 y (cdr-profile p)))
        (max y p)))
    (fit1 len p))

  (define (draw-list lst x y p)
    (draw-cons-cell x y)
    (let* ((tail (cdr lst))
           (tail-p (cdr-profile p))
           (new-p (cond ((null? tail)
                         (draw-nil x y)
                         tail-p)
                        ((pair? tail)
                         (draw-cdr-arrow x y 1)
                         (draw-list tail (+ x 1) y tail-p))
                        (else
                         (draw-cdr-arrow x y 1)
                         (mac#move arrow-space (quotient fontsize 2))
                         (mac#drawstring (object->string tail))
                         tail-p))))
      (draw-object (car lst) x y (cons (car-profile p) new-p))))

  (define (draw-object obj x y p)
    (if (pair? obj)
      (let ((len (object-length obj)))
        (let ((new-p (make-profile len p)))
          (let ((yy (car-profile new-p)))
            (draw-car-arrow x y (- yy y))
            (draw-list obj x yy new-p))))
      (let ((text (object->string obj)))
        (draw-car-arrow x y 1)
        (mac#move (- (quotient (mac#stringwidth text) 2)) fontsize)
        (mac#drawstring text)
        (make-profile 1 p))))

  (let ((w (mac#newwindow (mac#rect 40 10 250 500) "represent" #t)))
    (if (not (= w 0)) ; make sure it was created...
      (begin
        (mac#setport w)
        (mac#textfont 3) ; geneva
        (mac#textface 1) ; bold
        (mac#textsize fontsize)
        (draw-object obj 0 0 (initial-profile))
        (let loop () (if (not (mac#button)) (loop)))
        (mac#disposewindow w)))))

;============================================================================

; Event processing example
; ------------------------

; (read-keyboard echo?) returns the string of characters that are typed at
; the keyboard.  Input is terminated by the return key.  The characters are
; echoed if 'echo?' is true.
;
; Read-keyboard installs a special handler procedure to intercept the events
; that are produced by the toolbox routine 'GetNextEvent'.  Here, only
; keydown and autokey events are intercepted, but all events can be
; intercepted.
;
; Note: Because this procedure can be interrupted by the preemption clock,
; it is possible that some of the characters will be processed out of order.
; This is more noticeable in interpreted code than compiled code.  The wait
; loop is also a problem when interpreted because it causes frequent garbage
; collections.

(define (read-keyboard echo?)
  (let ((original-intercept-os-event ##intercept-os-event)
        (final-key-sequence #f)
        (key-sequence '()))

    (define (new-intercept-os-event event)
      (if (memv (mac#event-what event) '(3 5)) ; keydown or autokey
        (let ((key (modulo (mac#event-message event) 256)))
          (cond ((= key 13) ; return key?
                 (set! ##intercept-os-event original-intercept-os-event)
                 (set! final-key-sequence key-sequence))
                (else
                 (set! key-sequence (cons key key-sequence))
                 (if echo? (write-char (integer->char key)))))
          #t) ; #t = process other events
        (original-intercept-os-event event))) ; otherwise do normal processing

    (set! ##intercept-os-event new-intercept-os-event) ; setup the new handler

    (let loop () ; wait until final-key-sequence is known
      (let ((x final-key-sequence))
        (if (not x)
          (loop)
          (list->string (map integer->char (reverse x))))))))

;============================================================================

; Window example
; --------------

; (query prompt) pops up a window, displays the prompt and returns the
; expression typed in the window.  Note that a window is both an input and
; output port.
;
; try: (query "Enter value of 'n': ")

(define (query prompt)
  (let ((p (mac#open-window "query")))
    (display prompt p)
    (let ((x (read p)))
      (close-output-port p)
      x)))

; (pretty obj) pops up a window with the pretty printed representation
; of 'obj'.  Note that the result of 'pretty' is a port.  You should close
; this port to get rid of the window.
;
; try: (let ((p (pretty '(1 2 3)))) (read-keyboard #f) (close-output-port p))

(define (pretty obj)
  (let ((p (mac#open-window "pretty")))
    (pp obj p)
    p))

;============================================================================
